-----------------------------------------------------------------------------------------------
-- Client Lua Script for InfoWidgets, by lyravega
-- Copyright (c) NCsoft. All rights reserved
-----------------------------------------------------------------------------------------------
require "Window"
require "Apollo"
require "ApolloTimer"
require "GameLib"
require "GroupLib"
require "PlayerPathLib"
require "AbilityBook"
require "Money"
 
-----------------------------------------------------------------------------------------------
-- InfoWidgets Module Definition
-----------------------------------------------------------------------------------------------
local InfoWidgets = {}
local debugMode, debugLevel = true, 1

-----------------------------------------------------------------------------------------------
-- Local (Helper) Functions
-----------------------------------------------------------------------------------------------
local function t_copy(source, target, shallow)
	shallow = target == nil and false or shallow or false
	target = target or {}
    
	if shallow then for key, value in next, source do target[key] = value end
	else for key, value in next, source do
		if type(value) == "table" and type(target[key]) == "table" then t_copy(source[key], target[key])
		elseif type(value) == "table" then target[key] = t_copy(source[key])
		else target[key] = value end
	end end
	
	return target
end

local function t_update(target, source, shallow)
	if shallow then for key, value in next, target do target[key] = source[key] or value end
	else for key, value in next, target do
		if type(value) == "table" and type(source[key]) == "table" then t_update(target[key], source[key])
		elseif source[key] == nil then target[key] = value
		else target[key] = source[key] end
	end end
end

local function c_resize(offsetTable, width, height)
	return offsetTable[1], offsetTable[2], offsetTable[3]+width, offsetTable[4]+height
end

-----------------------------------------------------------------------------------------------
-- "Glocalization" & Upvalues
-----------------------------------------------------------------------------------------------
local eppg, epwc 

function InfoWidgets:_function_Glocalizer()
	eppg, epwc = GameLib.ElderPointsPerGem, GameLib.ElderPointsDailyMax
end

local Apollo, XmlDoc, GameLib, GroupLib, AbilityBook, PlayerPathLib, ApolloTimer = Apollo, XmlDoc, GameLib, GroupLib, AbilityBook, PlayerPathLib, ApolloTimer
local next = next
local t_unpack = unpack
local s_format = string.format
local m_floor = math.floor
 
-----------------------------------------------------------------------------------------------
-- Constants
-----------------------------------------------------------------------------------------------
local classIcons = {
	[GameLib.CodeEnumClass.Engineer] = 				"InfoWingetsSprites:Engineer_Colored",
	[GameLib.CodeEnumClass.Esper] = 					"InfoWingetsSprites:Esper_Colored",
	[GameLib.CodeEnumClass.Medic] = 					"InfoWingetsSprites:Medic_Colored",
	[GameLib.CodeEnumClass.Spellslinger] = 				"InfoWingetsSprites:Spellslinger_Colored",
	[GameLib.CodeEnumClass.Stalker] = 					"InfoWingetsSprites:Stalker_Colored",
	[GameLib.CodeEnumClass.Warrior] = 					"InfoWingetsSprites:Warrior_Colored",
}

local pathIcons = {
	[PlayerPathLib.PlayerPathType_Soldier] = 				"ClientSprites:Icon_Windows_UI_CRB_Soldier",
	[PlayerPathLib.PlayerPathType_Settler] = 				"ClientSprites:Icon_Windows_UI_CRB_Colonist",
	[PlayerPathLib.PlayerPathType_Scientist] = 				"ClientSprites:Icon_Windows_UI_CRB_Scientist",
	[PlayerPathLib.PlayerPathType_Explorer] = 				"ClientSprites:Icon_Windows_UI_CRB_Explorer",
}

local currency = {
	Renown = 									Money.CodeEnumCurrencyType.Renown,
	ElderGems = 									Money.CodeEnumCurrencyType.ElderGems,
	Prestige = 									Money.CodeEnumCurrencyType.Prestige,
	CraftingVouchers = 								Money.CodeEnumCurrencyType.CraftingVouchers,
}

local timerTable = { 
	meterTimer = 									0.25,
	delayedLoadTimer = 								0.5,
}
 
-----------------------------------------------------------------------------------------------
-- InfoWidgets Init & Load
-----------------------------------------------------------------------------------------------
function InfoWidgets:new(o)
	o = o or {}
	setmetatable(o, self)
	self.__index = self 

	o._tablePlayer = {}
	o._tableControls = {} --options menu controls
	o._tableWidgets = {
		progressWidgets = {},
		currencyWidgets = {},
		meterWidgets = {},
	}
	o._tableOptions = {
		widgets = {
			progressLeft = {
				enabled = true,
				type = "progressWidget",
				name = "progressLeft",
				track = "xp",
				buttonOnRight = false,
				compact = false,
				background = false,
				offsets = {-468, -19, -12, 2},
				dockOffsets = {-468, -19, -12, 2},
			},
			progressRight = {
				enabled = true,
				type = "progressWidget",
				name = "progressRight",
				track = "path",
				buttonOnRight = true,
				compact = false, 
				background = false,
				offsets = {12, -19, 468, 2},
				dockOffsets = {12, -19, 468, 2},
			},
			actionSet = {
				enabled = true,
				type = "actionSetWidget",
				name = "actionSet",
				customNames = {"Action Set 1", "Action Set 2", "Action Set 3", "Action Set 4", "Action Set 5", "Action Set 6"},
				background = false,
				offsets = {-611, -23, -496, 1},
				dockOffsets = {-611, -23, -496, 1},
			},
			currencyRenown = {
				enabled = true,
				type = "currencyWidget",
				name = "currencyRenown",
				currency = "Renown",
				background = false,
				offsets = {595, -24, 665, -4},
				dockOffsets = {595, -24, 665, -4},
			},
			currencyElderGems = {
				enabled = true,
				type = "currencyWidget",
				name = "currencyElderGems",
				currency = "ElderGems",
				background = false,
				offsets = {665, -24, 735, -4},
				dockOffsets = {665, -24, 735, -4},
			},
			currencyPrestige = {
				enabled = true,
				type = "currencyWidget",
				name = "currencyPrestige",
				currency = "Prestige",
				background = false,
				offsets = {805, -24, 875, -4},
				dockOffsets = {805, -24, 875, -4},
			},
			currencyCraftingVouchers = {
				enabled = true,
				type = "currencyWidget",
				name = "currencyCraftingVouchers",
				currency = "CraftingVouchers",
				background = false,
				offsets = {735, -24, 805, -4},
				dockOffsets = {735, -24, 805, -4},
			},
			meterFPS = {
				enabled = true,
				type = "meterWidget",
				name = "meterFPS",
				track = "FPS",
				background = false,
				offsets = {-428, -38, -370, -18},
				dockOffsets = {-428, -38, -370, -18},
			},
			meterLatency = {
				enabled = true,
				type = "meterWidget",
				name = "meterLatency",
				track = "Latency",
				background = false,
				offsets = {370, -38, 428, -18},
				dockOffsets = {370, -38, 428, -18},
			},
		},
		restoreDefaults = false,
		configVersion = 1,
	}
	o._tableInternal = {
		registeredEvents = {},
		requestReload = false,
	}

	return o
end

function InfoWidgets:Init()
	Apollo.RegisterAddon(self, true, "InfoWidgets", nil)
	
	Apollo.RegisterSlashCommand("infowidgets", 		"_windowFunction_ShowHelp", self)
	Apollo.RegisterSlashCommand("iw", 				"_windowFunction_ShowHelp", self)
end

function InfoWidgets:OnLoad()
	Apollo.LoadSprites("InfoWidgetsSprites.xml")

	self.xmlDocument = XmlDoc.CreateFromFile("InfoWidgets.xml")
	self.xmlDocument:RegisterCallback("OnDocLoaded", self)
	
	if debugMode and Apollo.GetAddon("Rover") then SendVarToRover("InfoWidgets", self) end
end

function InfoWidgets:GetAsyncLoadStatus()
	if Apollo.GetAddon("XPBar") then Apollo.SetConsoleVariable("hud.xpBarDisplay", 2) end
	if Apollo.GetAddon("XPBarExtended") then self._tableInternal.warning = true end
	
	return Apollo.AddonLoadStatus.Loaded
end
	
function InfoWidgets:OnDocLoaded()
	if self.xmlDocument and self.xmlDocument:IsLoaded() then
		self.help = Apollo.LoadForm(self.xmlDocument, "InfoWidgetsHelp", nil, self)
		self.help:Show(false, true)
		
		self._delayedLoadTimer = ApolloTimer.Create(timerTable.delayedLoadTimer, true, "OnDelayedLoad", self)
	end
end

function InfoWidgets:OnDelayedLoad()
	if not GameLib.GetPlayerUnit() then return else self._delayedLoadTimer:Stop() end
	
	if self._tableInternal.requestReload then RequestReloadUI() else self:_function_Glocalizer(); self._tableInternal.addOnInitialized = true end
	if self._tableInternal.warning then Apollo.LoadForm(self.xmlDocument, "Warning", nil, self); return end
	
	if PlayerPathLib.GetPathLevel() == 30 then self._tableInternal.maxPathLevel = true end
	if GameLib.GetPlayerUnit():GetBasicStats().nLevel == 50 then self._tableInternal.maxCharLevel = true end
	self._tableInternal.playerClass, self._tableInternal.playerPath = GameLib.GetPlayerUnit():GetClassId(), PlayerPathLib.GetPlayerPathType()
	
	for _, widgetAttributes in next, self._tableOptions.widgets do
		self:_function_NewWidget(widgetAttributes)
	end
end

-----------------------------------------------------------------------------------------------
-- InfoWidgets Save & Restore
-----------------------------------------------------------------------------------------------
function InfoWidgets:OnSave(saveDepth)
	if saveDepth ~= GameLib.CodeEnumAddonSaveLevel.Character then return nil end

	local savedVariables = t_copy(self._tableOptions)

	return savedVariables
end

function InfoWidgets:OnRestore(saveDepth, savedVariables) --OnLoad then GetAsyncLoad then OnRestore
	if saveDepth ~= GameLib.CodeEnumAddonSaveLevel.Character then return end
	
	if savedVariables then
		if savedVariables.configVersion ~= self._tableOptions.configVersion
		or savedVariables.restoreDefaults then self._tableOptions.restoreDefaults = false; self._tableInternal.requestReload = true
		else t_update(self._tableOptions, savedVariables) end 
	end
end

-----------------------------------------------------------------------------------------------
-- InfoWidgets Widget Update Functions & Handlers
-----------------------------------------------------------------------------------------------
function InfoWidgets:_function_UpdateCharXP(widgetTable)
	if self._tableInternal.maxCharLevel then self:_function_UpdateEP(widgetTable); return end

	local current, maximum = GetXp() - GetXpToCurrentLevel(), GetXpToNextLevel(); local percent = current/maximum*100; local tooltipTop = s_format("Level Progression: %.1f%% (%d/%d)", percent, current, maximum)
	local rested, projected = GetRestXp(), GetRestXpKillCreaturePool(); local restedPercent = rested/maximum*100; local tooltipBottom = rested > 0 and s_format("\nRested Pool: %d (%.1f%% of this level)", rested, restedPercent) or ""
	local Widget, Progress, Shadow, Button = widgetTable.Widget, widgetTable.children.Progress, widgetTable.children.Shadow, widgetTable.children.Button
	
	Widget:SetTooltip(tooltipTop..tooltipBottom)
	Progress:SetMax(maximum); Progress:SetProgress(current)
	Shadow:SetMax(maximum); Shadow:SetProgress(projected)
	Button:SetText(s_format("%.1f%%", percent))
end

function InfoWidgets:_function_UpdateEP(widgetTable)
	local current, maximum = GetElderPoints(), eppg; local percent = current/maximum*100; local tooltipTop = s_format("Elder Gem Progression: %.1f%% (%d/%d)", percent, current, maximum)
	local rested, projected = GetRestXp(), GetRestXpKillCreaturePool(); local restedPercent = rested/maximum*100; local tooltipBottom = rested > 0 and s_format("\nRested Pool: %d (%.1f%% of this gem)", rested, restedPercent) or ""
	local Widget, Progress, Shadow, Button = widgetTable.Widget, widgetTable.children.Progress, widgetTable.children.Shadow, widgetTable.children.Button
	
	Widget:SetTooltip(tooltipTop..tooltipBottom)
	Progress:SetMax(maximum); Progress:SetProgress(current)
	Shadow:SetMax(maximum); Shadow:SetProgress(projected)
	Button:SetText(s_format("%.1f%%", percent))
end

function InfoWidgets:_function_UpdatePathXP(widgetTable)
	if self._tableInternal.maxPathLevel then self:_function_UpdateEPWeekly(widgetTable); return end

	local current, maximum = PlayerPathLib.GetPathXPForLevel(), PlayerPathLib.GetPathXPForNextLevel(); local percent = current/maximum*100; local tooltip = s_format("Path Level Progression: %.1f%% (%d/%d)", percent, current, maximum)
	local Widget, Progress, Shadow, Button = widgetTable.Widget, widgetTable.children.Progress, widgetTable.children.Shadow, widgetTable.children.Button
	
	Widget:SetTooltip(tooltip)
	Progress:SetMax(maximum); Progress:SetProgress(current)
	Shadow:SetMax(0); Shadow:SetProgress(0)
	Button:SetText(s_format("%.1f%%", current/maximum*100))
end

function InfoWidgets:_function_UpdateEPWeekly(widgetTable)
	local current, maximum = GetPeriodicElderPoints()/eppg, epwc/eppg; local percent = m_floor(current/maximum*100); local tooltip = s_format("Weekly Elder Gem Progression: %.1f%% (%d/%d)", percent, current, maximum)
	local Widget, Progress, Shadow, Button = widgetTable.Widget, widgetTable.children.Progress, widgetTable.children.Shadow, widgetTable.children.Button
	
	Widget:SetTooltip(tooltip)
	Progress:SetMax(maximum); Progress:SetProgress(current)
	Shadow:SetMax(0); Shadow:SetProgress(0)
	Button:SetText(s_format("%d/140", current))
end

function InfoWidgets:_handler_UpdateProgress() --need to check if event passes any arguments
	for widgetName, widgetTable in next, self._tableWidgets.progressWidgets do
		local widgetTrack = widgetTable.attributes.track
		
		if widgetTrack == "xp" then 			self:_function_UpdateCharXP(widgetTable)
		elseif widgetTrack == "path" then 		self:_function_UpdatePathXP(widgetTable)
		elseif widgetTrack == "ep" then 		self:_function_UpdateEP(widgetTable)
		elseif widgetTrack == "epw" then 		self:_function_UpdateEPWeekly(widgetTable)
		end
	end
end

function InfoWidgets:_handler_SpecChange(actionSetIndex)
	self._tableWidgets.actionSetWidget.Widget:SetText(self._tableOptions.widgets.actionSet.customNames[actionSetIndex])
end

function InfoWidgets:_handler_CurrencyChange()
	for widgetName, widgetTable in next, self._tableWidgets.currencyWidgets do
		widgetTable.Widget:SetAmount(GameLib.GetPlayerCurrency(currency[widgetTable.attributes.currency]))
	end
end

function InfoWidgets:_handler_MeterTimer()
	for widgetName, widgetTable in next, self._tableWidgets.meterWidgets do
		local widgetTrack, WidgetDisplay = widgetTable.attributes.track, widgetTable.Widget
		
		if widgetTrack == "FPS" then 
			local fps = GameLib.GetFrameRate()
		
			WidgetDisplay:SetText(s_format("%.1f FPS", fps))
			WidgetDisplay:SetTextColor(fps > 30 and "UI_TextHoloBodyCyan" or "AddOnWarning")
		elseif widgetTrack == "Latency" then 
			local latency = GameLib.GetLatency()
			
			WidgetDisplay:SetText(s_format("%dms", latency))
			WidgetDisplay:SetTextColor(latency < 100 and "UI_TextHoloBodyCyan" or latency < 200 and "AddOnWarning" or "AddOnError")
		end
	end
end

-----------------------------------------------------------------------------------------------
-- InfoWidgets Widget Setup Functions
-----------------------------------------------------------------------------------------------
function InfoWidgets:_function_NewWidget(widgetAttributes)
	local widgetType, enabled = widgetAttributes.type, widgetAttributes.enabled
		
	if widgetType == "progressWidget" and enabled then self:_widgetSetup_Progress(widgetAttributes)
	elseif widgetType == "actionSetWidget" and enabled then self:_widgetSetup_ActionSet(widgetAttributes)
	elseif widgetType == "currencyWidget" and enabled then self:_widgetSetup_Currency(widgetAttributes)
	elseif widgetType == "meterWidget" and enabled then self:_widgetSetup_Meter(widgetAttributes)
	end
end

function InfoWidgets:_widgetSetup_Progress(widgetAttributes, Widget) --those variables start with a capital letter are XML based stuff in my codes, always.
	local Widget, widgetName = Widget or Apollo.LoadForm(self.xmlDocument, "ProgressWidget", "FixedHudStratum", self), widgetAttributes.name
	
	local widgetTable = {
		Widget = 			Widget, 
		attributes = 		widgetAttributes,
		children = {
			Progress = 	Widget:FindChild("Progress"), 
			Shadow = 	Widget:FindChild("Shadow"), 
			Button = 		Widget:FindChild("Button"), 
			Icon = 		Widget:FindChild("Icon"),
			Group = 		Widget:FindChild("Group"),
			Bar = 		Widget:FindChild("Bar"),
		},
	}; self._tableWidgets.progressWidgets[widgetName] = widgetTable; local widgetChildren, widgetTrack, widgetOffsets, Options = widgetTable.children, widgetAttributes.track, widgetAttributes.offsets, Widget:FindChild("Options")
	
	if widgetAttributes.compact then		Options:FindChild("compact"):SetCheck(true)
									widgetChildren.Bar:SetAnchorOffsets(70,0,456,21)
									widgetChildren.Group:SetAnchorOffsets(0,0,70,21)
									widgetChildren.Button:SetAnchorOffsets(-54,-21,7,0)
									widgetChildren.Icon:SetAnchorOffsets(-67,-18,-52,-3)
									c_resize(widgetAttributes.offsets,-386,0)
									widgetOffsets = {c_resize(widgetAttributes.offsets,-386,0)}						
	elseif widgetAttributes.buttonOnRight then	Options:FindChild("leftsided"):SetCheck(true)
									widgetChildren.Bar:SetAnchorOffsets(0,0,386,21)
									widgetChildren.Group:SetAnchorOffsets(386,0,456,21)
									widgetChildren.Button:SetAnchorOffsets(-77,-21,-16,0)
									widgetChildren.Icon:SetAnchorOffsets(-18,-18,-3,-3)
	else								widgetChildren.Bar:SetAnchorOffsets(70,0,456,21)
									widgetChildren.Group:SetAnchorOffsets(0,0,70,21)
									widgetChildren.Button:SetAnchorOffsets(-54,-21,7,0)
									widgetChildren.Icon:SetAnchorOffsets(-67,-18,-52,-3)
	end;								widgetChildren.Bar:Show(not widgetAttributes.compact); Options:FindChild("leftsided"):Enable(not widgetAttributes.compact)
	
	if widgetAttributes.background then 		Options:FindChild("background"):SetCheck(true); widgetChildren.Bar:SetSprite("BasicSprites:WhiteFill"); widgetChildren.Group:SetSprite("BasicSprites:WhiteFill") end
	
	if self._tableInternal.maxCharLevel then 	Options:FindChild("xp"):Enable(false) if widgetTrack == "xp" then self._tableOptions.widgets[widgetName].track = "ep"; widgetTrack = "ep" end 
	else 							Options:FindChild("ep"):Enable(false); Options:FindChild("epw"):Enable(false) end
	if self._tableInternal.maxPathLevel then	Options:FindChild("path"):Enable(false) if widgetTrack == "path" and self._tableInternal.maxCharLevel then self._tableOptions.widgets[widgetName].track = "epw"; widgetTrack = "epw" end end
	
	if widgetTrack == "xp" then 			widgetChildren.Icon:SetSprite(classIcons[self._tableInternal.playerClass]); self:_function_UpdateCharXP(widgetTable)
	elseif widgetTrack == "path" then 		widgetChildren.Icon:SetSprite(pathIcons[self._tableInternal.playerPath]); self:_function_UpdatePathXP(widgetTable)
	elseif widgetTrack == "ep" then 		widgetChildren.Icon:SetSprite("CRB_CurrencySprites:sprArtTier2"); self:_function_UpdateEP(widgetTable)
	elseif widgetTrack == "epw" then 		widgetChildren.Icon:SetSprite("CRB_CurrencySprites:sprArtTier2"); self:_function_UpdateEPWeekly(widgetTable)
	end; 							Options:FindChild(widgetTrack):SetCheck(true)
	
	Widget:SetData(widgetName)
	Widget:SetAnchorOffsets(t_unpack(widgetOffsets))
	
	if not self._tableInternal.registeredEvents.ExperienceGained then self._tableInternal.registeredEvents.ExperienceGained = true; Apollo.RegisterEventHandler("ExperienceGained", "_handler_UpdateProgress", self) end
	if not self._tableInternal.registeredEvents.ElderPointsGained then self._tableInternal.registeredEvents.ElderPointsGained = true; Apollo.RegisterEventHandler("ElderPointsGained", "_handler_UpdateProgress", self) end
	if not self._tableInternal.registeredEvents.UpdatePathXp then self._tableInternal.registeredEvents.UpdatePathXp = true; Apollo.RegisterEventHandler("UpdatePathXp", "_handler_UpdateProgress", self) end
end

function InfoWidgets:_widgetSetup_ActionSet(widgetAttributes)
	local Widget, widgetName = Apollo.LoadForm(self.xmlDocument, "ActionSetWidget", "FixedHudStratum", self), widgetAttributes.name
	
	local widgetTable = {
		Widget = 			Widget,
		attributes = 		widgetAttributes,
		children = {
			Next = 		Widget:FindChild("Next"),
			Previous = 	Widget:FindChild("Previous"),
		},
	}; self._tableWidgets.actionSetWidget = widgetTable; local Options = Widget:FindChild("Options")
	
	if widgetAttributes.background then 		Options:FindChild("background"):SetCheck(true); Widget:SetSprite("BasicSprites:WhiteFill") end
	
	for i=1, AbilityBook.GetNumUnlockedSpecs() do local actionSet = Options:FindChild(i); actionSet:SetText(widgetAttributes.customNames[i]); actionSet:Show(true) end
	
	Widget:SetText(self._tableOptions.widgets[widgetName].customNames[AbilityBook.GetCurrentSpec()])
	Widget:SetData(widgetName)
	Widget:SetAnchorOffsets(t_unpack(widgetAttributes.offsets))
	
	if not self._tableInternal.registeredEvents.SpecChanged then self._tableInternal.registeredEvents.SpecChanged = true; Apollo.RegisterEventHandler("SpecChanged", "_handler_SpecChange", self) end
end

function InfoWidgets:_widgetSetup_Currency(widgetAttributes)
	local Widget, widgetName = Apollo.LoadForm(self.xmlDocument, "CurrencyWidget", "FixedHudStratum", self), widgetAttributes.name
	
	local widgetTable = {
		Widget = 			Widget,
		attributes = 		widgetAttributes,
	}; self._tableWidgets.currencyWidgets[widgetName] = widgetTable; local Options = Widget:FindChild("Options")
	
	if widgetAttributes.background then 		Options:FindChild("background"):SetCheck(true); Widget:SetSprite("BasicSprites:WhiteFill") end
	
	Widget:SetAmount(GameLib.GetPlayerCurrency(currency[widgetAttributes.currency]))
	Widget:SetData(widgetName)
	Widget:SetAnchorOffsets(t_unpack(widgetAttributes.offsets))
	
	if not self._tableInternal.registeredEvents.PlayerCurrencyChanged then self._tableInternal.registeredEvents.currencyEvents = true; Apollo.RegisterEventHandler("PlayerCurrencyChanged", "_handler_CurrencyChange", self) end
end

function InfoWidgets:_widgetSetup_Meter(widgetAttributes)
	local Widget, widgetName = Apollo.LoadForm(self.xmlDocument, "MeterWidget", "FixedHudStratum", self), widgetAttributes.name

	local widgetTable = {
		Widget = 			Widget,
		attributes = 		widgetAttributes,
	}; self._tableWidgets.meterWidgets[widgetName] = widgetTable; local Options = Widget:FindChild("Options")
	
	if widgetAttributes.background then 		Options:FindChild("background"):SetCheck(true); Widget:SetSprite("BasicSprites:WhiteFill") end
	
	Widget:SetData(widgetName)
	Widget:SetAnchorOffsets(t_unpack(widgetAttributes.offsets))
	
	if not self._tableInternal.registeredEvents.MeterTimer then self._tableInternal.registeredEvents.MeterTimer = true; self._meterWidgetTimer = ApolloTimer.Create(timerTable.meterTimer, true, "_handler_MeterTimer", self) end
end

-----------------------------------------------------------------------------------------------
-- InfoWidgets Widget Functions
-----------------------------------------------------------------------------------------------

function InfoWidgets:_widgetFunction_WindowMove(_, widgetControl) --wndHandler, wndControl, nOldLeft, nOldTop, nOldRight, nOldBottom
	self._tableOptions.widgets[widgetControl:GetData()].offsets = {widgetControl:GetAnchorOffsets()}
end

function InfoWidgets:_widgetFunction_ActionSetWidget_ButtonSignal(_, widgetControl) --wndHandler, wndControl, eMouseButton
	local buttonName = widgetControl:GetName()

	if buttonName == "Next" then AbilityBook.NextSpec() 
	elseif buttonName == "Previous" then AbilityBook.PrevSpec() 
	end
end

function InfoWidgets:_widgetFunction_EditBoxReturn(_, widgetControl, text) --wndHandler, wndControl, strText
	local Widget, editBoxNumber = widgetControl:GetParent():GetParent(), tonumber(widgetControl:GetName())
	local widgetAttributes = self._tableOptions.widgets[Widget:GetData()]
	
	widgetAttributes.customNames[editBoxNumber] = text
	if editBoxNumber == AbilityBook.GetCurrentSpec() then Widget:SetText(text) end
end

local progressWidgetButtonFilter = {xp = true, path = true, ep = true, epw = true}
function InfoWidgets:_widgetFunction_ButtonCheck(_, widgetControl) --widgetData only has widgetName
	local Widget, toggleName = widgetControl:GetParent():GetParent(), widgetControl:GetName()
	local widgetAttributes, widgetName = self._tableOptions.widgets[Widget:GetData()], Widget:GetName()
	
	if toggleName == "background" and widgetName == "ProgressWidget" then widgetAttributes.background = true; Widget:FindChild("Bar"):SetSprite("BasicSprites:WhiteFill"); Widget:FindChild("Group"):SetSprite("BasicSprites:WhiteFill")
	elseif toggleName == "background" then widgetAttributes.background = true; Widget:SetSprite("BasicSprites:WhiteFill") 
	elseif toggleName == "compact" then widgetAttributes.compact = true; self:_widgetSetup_Progress(widgetAttributes, Widget)
	elseif toggleName == "leftsided" then widgetAttributes.buttonOnRight = true; self:_widgetSetup_Progress(widgetAttributes, Widget)
	elseif progressWidgetButtonFilter[toggleName] then widgetAttributes.track = toggleName; self:_widgetSetup_Progress(widgetAttributes, Widget)
	end
end

function InfoWidgets:_widgetFunction_ButtonUncheck(_, widgetControl)
	local Widget, toggleName = widgetControl:GetParent():GetParent(), widgetControl:GetName()
	local widgetAttributes = self._tableOptions.widgets[Widget:GetData()]
	
	if toggleName == "background" and Widget:GetName() == "ProgressWidget" then widgetAttributes.background = false; Widget:FindChild("Bar"):SetSprite(""); Widget:FindChild("Group"):SetSprite("")
	elseif toggleName == "background" then widgetAttributes.background = false; Widget:SetSprite("") 
	elseif toggleName == "compact" then widgetAttributes.compact = false; self:_widgetSetup_Progress(widgetAttributes, Widget)
	elseif toggleName == "leftsided" then widgetAttributes.buttonOnRight = false; self:_widgetSetup_Progress(widgetAttributes, Widget)
	end
end

local widgetNameFilter = {ProgressWidget = true, CurrencyWidget = true, ActionSetWidget = true, MeterWidget = true}
function InfoWidgets:_widgetFunction_MouseButtonUp(_, widgetControl, mouseButton) --wndHandler, wndControl, eMouseButton, nLastRelativeMouseX, nLastRelativeMouseY
	if mouseButton == 2 then local widgetName, Options = widgetControl:GetName(), widgetControl:FindChild("Options")
		if widgetNameFilter[widgetName] then Options:Show(not Options:IsShown(), false, 1) end
	end
end

function InfoWidgets:_widgetFunction_Dock(_, widgetControl)
	local Widget, Options = widgetControl:GetParent():GetParent(), widgetControl:GetParent()
	local widgetAttributes = self._tableOptions.widgets[Widget:GetData()]
	
	widgetAttributes.offsets = widgetAttributes.dockOffsets
	Widget:SetAnchorOffsets(t_unpack(widgetAttributes.dockOffsets))
	Options:Show(false, false, 1)
	ChatSystemLib.PostOnChannel( ChatSystemLib.ChatChannel_Datachron, "Widget has been docked to saved dock coordinates", "InfoWidgets")
end

function InfoWidgets:_widgetFunction_Disable(_, widgetControl)
	local Widget = widgetControl:GetParent():GetParent()
	local widgetAttributes = self._tableOptions.widgets[Widget:GetData()]
	
	widgetAttributes.enabled = false
	Widget:Show(false, false, 1)
	ChatSystemLib.PostOnChannel( ChatSystemLib.ChatChannel_Datachron, "Widget has been disabled", "InfoWidgets")
end

function InfoWidgets:_widgetFunction_Save(_, widgetControl)
	local Widget, Options = widgetControl:GetParent():GetParent(), widgetControl:GetParent()
	local widgetAttributes = self._tableOptions.widgets[Widget:GetData()]
	
	widgetAttributes.dockOffsets = widgetAttributes.offsets
	Options:Show(false, false, 1)
	ChatSystemLib.PostOnChannel( ChatSystemLib.ChatChannel_Datachron, "New dock offsets has been saved", "InfoWidgets")
end

function InfoWidgets:OnConfigure() self:_windowFunction_ShowHelp() end
function InfoWidgets:_windowFunction_ShowHelp() self.help:Invoke() end
function InfoWidgets:_windowFunction_Cancel() self.help:Show(false) end

function InfoWidgets:_windowFunction_RestoreDefaults()
	self._tableOptions.restoreDefaults = true

	RequestReloadUI()
end

function InfoWidgets:_windowFunction_RestoreDisabled()
	for _, widgetAttributes in next, self._tableOptions.widgets do
		if not widgetAttributes.enabled then widgetAttributes.enabled = true end
	end
	
	RequestReloadUI()
end

-----------------------------------------------------------------------------------------------
-- InfoWidgets Instance
-----------------------------------------------------------------------------------------------
local InfoWidgetsInst = InfoWidgets:new()
InfoWidgetsInst:Init()
